'' =================================================================================================
''
''   File....... 
''   Purpose....
''   Author..... Author
''               Copyright (c) 2020 Author
''               -- see below for terms of use
''   E-mail.....  
''   Started....
''   Updated....
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  BR_TERM  = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                             
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 


con

  #true,  ON, OFF 
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  prng : "jm_prng"                                              '   pseudo-random number generator
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded

                                                                 
dat { pre-initialized }


var { globals }                                                              
                                                                 
                                                                 
pub main
                                                                 
  setup                                                          
                                                                 
  wait_for_terminal(true)
                
  term.str(string("Hello, World!", 13, 13))            
                                                                 
  repeat                                                         
    waitcnt(0)


pub wait_for_terminal(clear)

  ' Download to RAM with F10
  ' F12 to open PST
  ' Click [Enable]
  ' Press Enter

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)

                                                               
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $CAFE_BABE, cnt <- 2, cnt)         ' seed randomizer                                                                           

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, BR_TERM)                          ' start serial for terminal *

                   
con { license }                                                  

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}     